//
//   @file    ploughshare.cxx         
//   
//
//   @author M Sutton
//
//   $Id: ploughshare.cxx, v0.0   Sun  6 May 2018 00:40:14 CEST sutt $
//
//   Copyright (C) 2018 M Sutton (sutt@cern.ch)    


#include <cstdlib>

#include <dirent.h>
#include <sys/stat.h>

#include "amconfig.h"
#include "ploughshare.h"



bool file_exists( const std::string& filename ) { 
  struct stat sb;
  if ( stat( filename.c_str(), &sb)==0 ) return true; // && S_ISREG(sb.st_mode ))
  else return false;
}




/// Split a string into tokens at expr
 
std::vector<std::string> split( const std::string& _s, const std::string& expr ) {     
  std::string s = _s;
  std::vector<std::string> tokens;
  //  if ( s == "" ) return tokens;
  size_t pos = s.find(expr);
  while ( pos != std::string::npos ) { 
    std::string frag = s.substr( 0, pos );
    s.erase( 0, pos+expr.size() );
    tokens.push_back( frag );
    pos = s.find(expr);
  } 
  tokens.push_back(s);
  return tokens;
}



std::string strip( std::string s, const std::string expr, bool till_end=false ) { 
  size_t tpos = s.find( expr );
  if ( tpos != std::string::npos ) { 
    if ( !till_end ) s.erase( tpos, expr.size() );
    else             s.erase( tpos, s.size()-tpos );
  }  
  return s;
}



int ploughshare::init() { 

  /// read the path for the local ploughshare repository
  
  char* _path = getenv( "PLOUGHSHARE_PATH" );

  std::string ploughshare_path = "";

  if ( _path != 0 ) ploughshare_path = std::string(_path);

  /// if PLOUGHSHARE_PATH env variable not set, 
  /// use the default path from the installation
  if ( ploughshare_path != "" ) mpath = split( ploughshare_path, ":" );
  else                          mpath.push_back( DATADIR );

  for ( size_t i=0 ; i<mpath.size() ; i++ ) std::cout << i << " " << mpath[i] << std::endl; 

  return 0;
}



std::string group( std::string s ) { 
  std::vector<std::string> parsed = split( s, "-" );
  if ( parsed.empty() ) return "";
  return parsed[0];
}



int ploughshare::download( const std::string& url ) {
  std::cout << "\033[0;32m fetching: " << url << " \033[0m" << std::endl;
  if ( mverbose ) return system( (std::string( "wget -N -P /tmp " ) + url ).c_str() );
  else            return system( (std::string( "wget -N -P /tmp " ) + url + " &> /dev/null" ).c_str() );
}





size_t ploughshare::fetch( const std::string& data ) { 

  if ( data == "" ) return 0;
 
  std::string grp = group( data );

  /// throw an exception here ???
  if ( grp == "" ) return 0;

  std::string dataset = data;

  dataset = strip( dataset, ".tgz" );
  dataset = strip( dataset, ".tar.tgz" );

  std::string file = "";

  bool single = false;

  std::string url = std::string(PACKAGE_URL) + "/db/" + grp + "/";

  if ( dataset.find( "-xsec" )!=std::string::npos ) { 
     dataset = strip( dataset, "-xsec", true );
     file =  data;
     url +=  dataset + "/grids/";
     single = true;
  }
  else {
    url += dataset + "/";
    file = dataset + ".tgz"; 
  }

  url += file;

  int status = 0;

  std::string dir = mpath[0] + "/" + grp;
    
  /// check if something has already been download
  /// NB: Need to somehow keep track of how many files there are 
  ///     in total in this directory so that we can avoid downloading 
  ///     them if they are already there, but will still download them
  ///     if there are some - but not all - the files in that directory

  if ( file_exists( dir + "/" + dataset + "/grids" ) ) { 
    std::cout << "\033[0;32m already downloaded: " << dataset << "\033[0m" << std::endl;
  }
  else {

    system( (std::string("mkdir -p ") + dir + "/" + dataset + "/grids").c_str() );

    /// now do the download ...
    
    if ( url != "" && (download( url ) == 0 ) )  { 
      
      std::string cmd = "";
      
      if ( single ) cmd = std::string("cp /tmp/")+file+ " " + dir + "/" + dataset + "/grids" ;    
      else          cmd = std::string("tar -xzf /tmp/")+file + " -C " + dir; 
      
      status = system( cmd.c_str() ); 
      
    }
    
  }

 
  std::set<std::string> grids = find( grp + "/" + dataset + "/grids" );

  if ( grids.size() ) { 

    //  mmap.insert(  std::map<std::string, std::set<std::string> >::value_type( dataset, grids ) );
    //  mmap.insert( std::map<std::string, ploughshare::dataset_t>::value_type( dataset, ploughshare::dataset_t(dataset, grp+"/"+dataset, grids) ) );

    std::string dpath = *grids.begin()+"/"+grp+"/"+dataset;
    
    grids.erase(grids.begin());

    mmap.insert( std::map<std::string, ploughshare::dataset_t>::value_type( dataset, ploughshare::dataset_t(dataset, dpath, grids) ) );
    
    mgrids.insert( grids.begin(), grids.end() );
    
    //  if ( grids.size() ) std::cout << "\t" << dataset << " contains:\n" << grids << std::endl;

    //    status = 1;

    return grids.size();
  }

  //  else status = 0; 

  return 0;

}





const std::set<std::string> ploughshare::find( const std::string& dataset ) const { 

  std::set<std::string>  sgrids;

  for ( size_t i=0 ; i<mpath.size() ; i++ ) { 

    //    std::cout << i << "\t" << mpath[i] << std::endl; 

    std::string fpath = mpath[i] + "/" + dataset;

    DIR* dir;

    //    std::cout << "path " << fpath << std::endl; 

    if ( (dir=opendir(fpath.c_str()))!=0 ) {

      struct dirent *ent;

      while ( (ent=readdir(dir))!=0 ) {
	
	std::string name = ent->d_name;

	if ( name == "." || name == ".." ) continue;

	//	if ( contains ) std::cout << dataset << " contains:\n";  
	//	std::cout << "\t" << ( fpath + "/" + name ) << std::endl; 

	if ( sgrids.size() == 0 ) sgrids.insert(mpath[i]);

	sgrids.insert( fpath+"/"+name );

      }
      closedir(dir);
    }

  }

  return sgrids;
}


